import { z } from "zod";
import axios from "axios";

import { createStep } from "../create-step";

/**
 * Executes a JSON HTTP Request.
 */
export default createStep({
  id: "std::HTTPRequest",
  input: z.object({
    headers: z.record(z.string(), z.string()).optional(),
    url: z.string(),
    method: z.enum(["DELETE", "GET", "PATCH", "POST", "PUT"]),
  }),
  output: z.object({
    statusCode: z.number().int(),
    data: z.string(),
  }),
  handle: async ({ url, method, headers }) => {
    const response = await axios.request({
      url: url,
      method: method,
      headers: {
        ...headers,
        // TODO: how to support different content types?
        "Content-Type": "application/json",
      },
    });

    return {
      statusCode: response.status,
      data: response.data,
    };
  },
});
