import fs from "fs";
import path from "path";
import { isStep, Step } from "./step";

export interface PluginPackageProps {
  /**
   * Folder to recursively load steps from.
   */
  folder: string;
}

export interface GlueStikPlugin {
  /**
   * The {@link Step} that constitutes this {@link GlueStikPlugin}
   *
   * note: we might eventually have different types here.
   */
  step: Step;
  /**
   * The file that contains this {@link GlueStikPlugin}.
   */
  file: string;
}

/**
 * Loads {@link Step}s from a folder.
 */
export class PluginPackage {
  /**
   * Folder tos cane for all Plugins.
   */
  readonly folder: string;

  /**
   * Cached value of loaded plugins.
   */
  #plugins: GlueStikPlugin[] | undefined;

  /**
   * List of all {@link GlueStikPlugin}s in this {@link PluginPackage}.
   */
  get plugins(): GlueStikPlugin[] {
    return this.#plugins ?? (this.#plugins = loadPlugins(this.folder));
  }

  constructor(props: PluginPackageProps) {
    this.folder = props.folder;
  }
}

/**
 * Recursively load {@link Step}s and {@link StepSpec}s from the file sytem.
 *
 * TODO: async?
 */
function loadPlugins(folder: string): GlueStikPlugin[] {
  const files = fs.readdirSync(folder).map((file) => path.join(folder, file));

  return files.flatMap((file) => {
    const stat = fs.statSync(file);
    if (stat.isDirectory()) {
      return loadPlugins(file);
    } else if (file.endsWith(".js") && stat.isFile()) {
      const step = require(file).default;
      if (isStep(step)) {
        return [
          {
            step,
            file: file,
          },
        ];
      }
    }
    return [];
  });
}
