import z from "zod";

export function isStep(a: any): a is Step {
  return (
    a &&
    typeof a === "object" &&
    typeof a.id === "string" &&
    typeof a.handle === "function" &&
    "input" in a &&
    "output" in a
  );
}

/**
 * The data model of a discrete step that is available within the Gluestik Platform.
 */
export interface Step<In = any, Out = any> {
  /**
   * Unique identifier. This ID uniquely identifies this workflow
   * across the entire system.
   */
  id: string;
  /**
   * An optional description of this workflow.
   */
  description?: string;
  /**
   * The input schema for the workflow.
   */
  input: z.ZodType<In>;
  /**
   * The output schema for the workflow.
   */
  output: z.ZodType<Out>;
  /**
   * A function that builds the {@link Workflow}'s logic using the {@link Flow} DSL.
   */
  handle(input: In): Promise<Out>;
}
